@@ -2,9 +2,10 @@
 import json
 import logging
 import re
+import types
 from copy import deepcopy
 from pathlib import Path
-from typing import List
+from typing import List, Union
 
 import gensim.downloader
 from gensim.models import FastText, Word2Vec
@@ -19,10 +20,9 @@ class Conceptualizer:
     def __init__(
         self,
         nlp: Language,
-        name: str,
-        data: dict,
+        data: dict = {},
+        model: Union[str, FastText, KeyedVectors, Word2Vec] = None,
         topn: list = None,
-        model_path: str = None,
         word_delimiter: str = "_",
         ent_score: bool = False,
         exclude_pos: list = None,
@@ -31,6 +31,7 @@ class Conceptualizer:
         case_sensitive: bool = False,
         json_path: str = "./matching_patterns.json",
         verbose: bool = True,
+        name: str = "concise_concepts",
     ):
         """
         The function takes in a dictionary of words and their synonyms, and then creates a new dictionary of words and
@@ -55,6 +56,7 @@ class Conceptualizer:
             if the entity is "New York", it will also include "New York City" as an entity, defaults to False (optional)
         :param case_sensitive: Whether to match the case of the words in the text, defaults to False (optional)
         """
+        assert data, ValueError("You must provide a dictionary of words to match")
         self.verbose = verbose
         self.log_cache = {"key": list(), "word": list(), "word_key": list()}
         if Span.has_extension("ent_score"):
@@ -62,14 +64,40 @@ class Conceptualizer:
         if ent_score:
             Span.set_extension("ent_score", default=None)
         self.ent_score = ent_score
-        self.orignal_words = [j for i in data.values() for j in i]
-        self.original_data = deepcopy(data)
         self.data = data
         self.name = name
         self.nlp = nlp
         self.topn = topn
-        self.model_path = model_path
+        self.model = model
         self.match_rule = {}
+        self.set_exclude_pos(exclude_pos)
+        self.set_exclude_dep(exclude_dep)
+        self.json_path = json_path
+        self.include_compound_words = include_compound_words
+        self.case_sensitive = case_sensitive
+        self.word_delimiter = word_delimiter
+        if "lemmatizer" not in self.nlp.component_names:
+            logger.warning(
+                "No lemmatizer found in spacy pipeline. Consider adding it for matching"
+                " on LEMMA instead of exact text."
+            )
+            self.match_key = "ORTH"
+        else:
+            self.match_key = "LEMMA"
+        if "entity_ruler" in self.nlp.component_names:
+            logger.warning(
+                "Entity Ruler already exists in the pipeline. Removing old rulers"
+            )
+            self.nlp.remove_pipe("entity_ruler")
+        self.run()
+
+    def set_exclude_dep(self, exclude_dep: list):
+        if exclude_dep is None:
+            exclude_dep = []
+        if exclude_dep:
+            self.match_rule["DEP"] = {"NOT_IN": exclude_dep}
+
+    def set_exclude_pos(self, exclude_pos: list):
         if exclude_pos is None:
             exclude_pos = [
                 "VERB",
@@ -85,24 +113,6 @@ class Conceptualizer:
             ]
         if exclude_pos:
             self.match_rule["POS"] = {"NOT_IN": exclude_pos}
-        if exclude_dep is None:
-            exclude_dep = []
-        if exclude_dep:
-            self.match_rule["DEP"] = {"NOT_IN": exclude_dep}
-        self.json_path = json_path
-        self.check_validity_path()
-        self.include_compound_words = include_compound_words
-        self.case_sensitive = case_sensitive
-        self.word_delimiter = word_delimiter
-        if "lemmatizer" not in self.nlp.component_names:
-            logger.warning(
-                "No lemmatizer found in spacy pipeline. Consider adding it for matching"
-                " on LEMMA instead of exact text."
-            )
-            self.match_key = "ORTH"
-        else:
-            self.match_key = "LEMMA"
-        self.run()
 
     def run(self) -> None:
         self.check_validity_path()
@@ -146,7 +156,7 @@ class Conceptualizer:
         If the user doesn't specify a topn value for each class,
         then the topn value for each class is set to 100
         """
-        if not self.topn:
+        if self.topn is None:
             self.topn_dict = {key: 100 for key in self.data}
         else:
             num_classes = len(self.data)
@@ -161,24 +171,34 @@ class Conceptualizer:
         If it's not a valid path, then we raise an exception.
         If the model_path is None, then we load the model from the internal embeddings of the spacy model
         """
-        if self.model_path:
-            available_models = gensim.downloader.info()["models"]
-            if self.model_path in available_models:
-                self.kv = gensim.downloader.load(self.model_path)
-            else:
-                try:
-                    self.kv = FastText.load(self.model_path).wv
-                except Exception as e1:
+        if isinstance(self.model, str):
+            if self.model:
+                available_models = gensim.downloader.info()["models"]
+                if self.model in available_models:
+                    self.kv = gensim.downloader.load(self.model)
+                else:
                     try:
-                        self.kv = Word2Vec.load(self.model_path).wv
-                    except Exception as e2:
+                        self.kv = FastText.load(self.model).wv
+                    except Exception as e1:
                         try:
-                            self.kv = KeyedVectors.load(self.model_path)
-                        except Exception as e3:
-                            raise Exception(
-                                "Not a valid gensim model. FastText, Word2Vec,"
-                                f" KeyedVectors.\n {e1}\n {e2}\n {e3}"
-                            )
+                            self.kv = Word2Vec.load(self.model).wv
+                        except Exception as e2:
+                            try:
+                                self.kv = KeyedVectors.load(self.model)
+                            except Exception as e3:
+                                try:
+                                    self.kv = KeyedVectors.load_word2vec_format(
+                                        self.model, binary=True
+                                    )
+                                except Exception as e4:
+                                    raise Exception(
+                                        "Not a valid gensim model. FastText, Word2Vec,"
+                                        f" KeyedVectors.\n {e1}\n {e2}\n {e3}\n {e4}"
+                                    )
+        elif isinstance(self.model, (FastText, Word2Vec)):
+            self.kv = self.model.wv
+        elif isinstance(self.model, KeyedVectors):
+            self.kv = self.model
         else:
             wordList = []
             vectorList = []
@@ -230,15 +250,13 @@ class Conceptualizer:
                 else:
                     raise Exception(msg)
         self.data = deepcopy(verified_data)
+        self.original_data = deepcopy(verified_data)
 
     def expand_concepts(self) -> None:
         """
         For each key in the data dictionary, find the topn most similar words to the key and the values in the data
         dictionary, and add those words to the values in the data dictionary
         """
-
-        self.original_data = deepcopy(self.data)
-
         for key in self.data:
             present_key = self._check_presence_vocab(key)
             if present_key:
@@ -403,8 +421,12 @@ class Conceptualizer:
         :param doc: Doc
         :type doc: Doc
         """
-        if self.ent_score:
-            doc = self.assign_score_to_entities(doc)
+        if isinstance(doc, str):
+            doc = self.nlp(doc)
+        elif isinstance(doc, Doc):
+            if self.ent_score:
+                doc = self.assign_score_to_entities(doc)
+
         return doc
 
     def pipe(self, stream, batch_size=128) -> Doc:
@@ -415,6 +437,12 @@ class Conceptualizer:
         :param stream: a generator of documents
         :param batch_size: The number of documents to be processed at a time, defaults to 128 (optional)
         """
+        if isinstance(stream, str):
+            stream = [stream]
+
+        if not isinstance(stream, types.GeneratorType):
+            stream = self.nlp.pipe(stream, batch_size=batch_size)
+
         for docs in util.minibatch(stream, size=batch_size):
             for doc in docs:
                 if self.ent_score:
